--_________________________________________________________________________________________________
--                                                                                                |
--                                         |The nanoFIP|                                          |
--                                                                                                |
--                                         CERN,BE/CO-HT                                          |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                         masterfip_rx                                           |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         masterfip_rx.vhd                                                                  |
--                                                                                                |
-- Description  The unit groups the main actions that regard FIELDRIVE data reception.            |
--              It instantiates the units/processes:                                              |
--                                                                                                |
--              o wf_rx_osc         : for the clock recovery                                      |
--                                                                                                |
--              o wf_rx_deglitcher  : for the filtering of the input FD_RXD                       |
--                                                                                                |
--              o wf_rx_deserializer: for the bytes retrieval; also detects FSS/FES & checks CRC  |
--                                                                                                |
--              o bytes_c           : for the counting of the retrieved bytes                     |
--                                                                                                |
--              o create_32bit_words: for the formation of 32-bit words to be provided to the WRNC|
--                                                                                                |
--                                           WRNC                                                 |
--                     ___________________________________________________________                |
--                    |                                                           |               |
--                    |   _________       _______________________                 |               |
--                    |  |         |     |                       |                |               |
--                    |  | bytes_c |     |   create_32bit_words  |                |               |
--                    |  |_________|     |_______________________|                |               |
--                    |      /\                     /\                 _________  |               |
--                    |   _______________________________________     |         | |               |
--                    |  |                                       |    |         | |               |
--                    |  |           wf_rx_deserializer          |    |  wf_rx  | |               |
--                    |  |                                       |  < |  _osc   | |               |
--                    |  |_______________________________________|    |         | |               |
--                    |                     /\                        |_________| |               |
--                    |   _______________________________________                 |               |
--                    |  |                                       |                |               |
--                    |  |            wf_rx_deglitcher           |                |               |
--                    |  |_______________________________________|                |               |
--                    |                                                           |               |
--                    |___________________________________________________________|               |
--                                                 /\                                             |
--              ___________________________________________________________________               |
--              0_____________________________FIELDBUS______________________________0             |
--                                                                                                |
--                                                                                                |
-- Authors      Evangelia Gousiou     (Evangelia.Gousiou@cern.ch)                                 |
-- Date         30/09/2015                                                                        |
-- Version      v0.01                                                                             |
-- Depends on                                                                                     |
--                                                                                                |
----------------                                                                                  |
-- Last changes                                                                                   |
--     09/2015  v0.01  EG  First version                                                          |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                               GNU LESSER GENERAL PUBLIC LICENSE                                |
--                              ------------------------------------                              |
-- This source file is free software; you can redistribute it and/or modify it under the terms of |
-- the GNU Lesser General Public License as published by the Free Software Foundation; either     |
-- version 2.1 of the License, or (at your option) any later version.                             |
-- This source is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;       |
-- without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      |
-- See the GNU Lesser General Public License for more details.                                    |
-- You should have received a copy of the GNU Lesser General Public License along with this       |
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html                     |
---------------------------------------------------------------------------------------------------



--=================================================================================================
--                                       Libraries & Packages
--=================================================================================================

-- Standard library
library IEEE;
use IEEE.STD_LOGIC_1164.all; -- std_logic definitions
use IEEE.NUMERIC_STD.all;    -- conversion functions
-- Specific library
library work;
use work.masterFIP_pkg.all;  -- definitions of types, constants, entities
use work.wf_package.all;


--=================================================================================================
--                              Entity declaration for masterfip_rx
--=================================================================================================
entity masterfip_rx is port(
  -- INPUTS
    clk_40m_i             : in std_logic;                     -- 40 MHZ clock

    rst_i                 : in std_logic;                     -- core rst, synched with clk_40m_i
    
    rx_rst_i              : in std_logic;                     -- rx reset during production or
                                                              -- reset pulse from the WRNC when during
                                                              -- reception a frame is rejected
                                                              -- (ex: RP_DAT > 133 bytes, wrong CTRL byte)
                                           
    speed_i               : in std_logic_vector(1 downto 0);  -- WorldFIP bit rate

    fd_rxd_a_i            : in std_logic;                     -- FielDrive receiver data


  -- OUTPUTS
    rx_byte_ready_p_o     : out std_logic;                    -- pulse indicating a new retrieved byte

    rx_byte_o             : out std_logic_vector(7 downto 0); -- currently retrieved byte

    rx_byte_index_o       : out std_logic_vector(8 downto 0); -- index of currently retrieved byte
                                                              -- counting starts after FSS; it includes the
                                                              -- CTRL, DATA, CRC and FES fields

    rx_ctrl_byte_o        : out std_logic_vector(7 downto 0); -- frame CTRL byte

    rx_frame_o            : out rx_frame_t;                   -- frame DATA bytes
                                                              -- structure with 66 words of 32-bit each
                                                              -- this represents the max length of a frame
                                                              -- upon the rx_fss_crc_fes_ok_p_o only
                                                              -- rx_byte_index_o bytes have to be read by the WRNC

    rx_fss_crc_fes_ok_p_o : out std_logic;                    -- indication of a frame with
                                                              -- correct FSS, FES & CRC; pulse upon FES detection
                                           
    rx_crc_wrong_p_o      : out std_logic;                    -- indication of a frame with
                                                              -- wrong CRC; pulse upon FES detection

    rx_fss_received_p_o   : out std_logic);                   -- pulse upon FSS detection (ID/ RP_DAT)

end entity masterfip_rx;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================
architecture struc of masterfip_rx is

  -- wf_rx_osc
  signal s_rx_osc_rst, s_adjac_bits_window, s_signif_edge_window : std_logic;
  signal s_sample_bit_p, s_sample_manch_bit_p                    : std_logic;
  -- wf_rx_deglitcher
  signal s_fd_rxd_filt, s_rxd_filt_edge_p                        : std_logic;
  signal s_fd_rxd_filt_f_edge_p, s_fd_rxd_filt_r_edge_p          : std_logic;
  -- wf_rx_deserializer  
  signal rx_byte_ready_p                                         : std_logic;
  signal rx_byte                                                 : std_logic_vector  (7 downto 0);
  -- retreived bytes into 32-bit regs
  signal byte0, byte1, byte2, byte3                              : std_logic_vector(7 downto 0);
  signal word32                                                  : std_logic_vector(31 downto 0);
  signal word32_num                                              : integer range 0 to 31;
  -- bytes counter
  signal rx_byte_index, rx_byte_index_d1                         : unsigned(8 downto 0);
  signal bytes_c_rst                                             : std_logic;
  

--=================================================================================================
--                                       architecture begin
--=================================================================================================
begin

---------------------------------------------------------------------------------------------------
--                            Deserializer as in the nanoFIP project                             --
---------------------------------------------------------------------------------------------------

  cmp_rx_deglitcher: wf_rx_deglitcher
  port map(
    uclk_i                 => clk_40m_i,
    nfip_rst_i             => rst_i,
    fd_rxd_a_i             => fd_rxd_a_i,
  -----------------------------------------------------------------
    fd_rxd_filt_o          => s_fd_rxd_filt,
    fd_rxd_filt_edge_p_o   => s_rxd_filt_edge_p,
    fd_rxd_filt_f_edge_p_o => s_fd_rxd_filt_f_edge_p);
  -----------------------------------------------------------------
    s_fd_rxd_filt_r_edge_p <= s_rxd_filt_edge_p and (not s_fd_rxd_filt_f_edge_p);

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --    
  cmp_rx_deserializer: wf_rx_deserializer
  port map(
    uclk_i                  => clk_40m_i,
    nfip_rst_i              => rst_i,
    rx_rst_i                => rx_rst_i,
    fd_rxd_f_edge_p_i       => s_fd_rxd_filt_f_edge_p,
    fd_rxd_r_edge_p_i       => s_fd_rxd_filt_r_edge_p,
    fd_rxd_i                => s_fd_rxd_filt,
    sample_bit_p_i          => s_sample_bit_p,
    sample_manch_bit_p_i    => s_sample_manch_bit_p,
    signif_edge_window_i    => s_signif_edge_window,
    adjac_bits_window_i     => s_adjac_bits_window,
  -----------------------------------------------------------------
    byte_o                  => rx_byte,
    byte_ready_p_o          => rx_byte_ready_p,
    fss_crc_fes_ok_p_o      => rx_fss_crc_fes_ok_p_o,
    rx_osc_rst_o            => s_rx_osc_rst,
    fss_received_p_o        => rx_fss_received_p_o,
    crc_wrong_p_o           => rx_crc_wrong_p_o);
  -----------------------------------------------------------------
  rx_byte_ready_p_o         <= rx_byte_ready_p;
  rx_byte_o                 <= rx_byte;

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --   
  cmp_rx_osc: wf_rx_osc
  port map(
    uclk_i                  => clk_40m_i,
    rate_i                  => speed_i,
    nfip_rst_i              => rst_i,
    fd_rxd_edge_p_i         => s_rxd_filt_edge_p,
    rx_osc_rst_i            => s_rx_osc_rst,
  -----------------------------------------------------------------
    rx_manch_clk_p_o        => s_sample_manch_bit_p,
    rx_bit_clk_p_o          => s_sample_bit_p,
    rx_signif_edge_window_o => s_signif_edge_window,
    rx_adjac_bits_window_o  => s_adjac_bits_window);
  -----------------------------------------------------------------


---------------------------------------------------------------------------------------------------
--                                         bytes counter                                         --
---------------------------------------------------------------------------------------------------
  cmp_rx_bytes_cnt: wf_incr_counter
  generic map(g_counter_lgth => 9)
  port map(
    uclk_i            => clk_40m_i,
    counter_reinit_i  => bytes_c_rst,
    counter_incr_i    => rx_byte_ready_p,
    counter_is_full_o => open,
    -------------------------------------------------------
    counter_o         => rx_byte_index);
    -------------------------------------------------------
  bytes_c_rst         <= '1' when (rst_i = '1' or rx_rst_i = '1') else '0';
  rx_byte_index_o     <= std_logic_vector(rx_byte_index);

  
---------------------------------------------------------------------------------------------------
--                      combination of four retrieved bytes to a 32-bit word                     --
---------------------------------------------------------------------------------------------------   
  create_32bit_words: process (clk_40m_i)
  begin
    if rising_edge (clk_40m_i) then
      if rst_i = '1' or rx_rst_i = '1' then
        byte0   <= (others => '0');
        byte1   <= (others => '0');
        byte2   <= (others => '0');
        byte3   <= (others => '0');        
        rx_ctrl_byte_o <= (others  => '0');  
        else
        if rx_byte_ready_p = '1' then
          if rx_byte_index = resize(unsigned(c_CTRL_BYTE_INDEX),9) then
            rx_ctrl_byte_o <= rx_byte;    
          else            
            byte0          <= rx_byte;
            byte1          <= byte0;
            byte2          <= byte1;
            byte3          <= byte2;
          end if;
        end if;
      end if;
    end if;
   end process;
  -------------------------------------------
  word32 <= byte0 & byte1 & byte2 & byte3;

  
---------------------------------------------------------------------------------------------------
--                        Storage of consumed bytes to the cons_frame regs                       --
---------------------------------------------------------------------------------------------------
-- transfer 32bit words to the cons_frame registers

  delay: process (clk_40m_i)
  begin
    if rising_edge (clk_40m_i) then
      if rst_i = '1' or rx_rst_i = '1' then
        rx_byte_index_d1   <= (others =>'0'); -- needed for synching
      else
        if rx_byte_ready_p = '1' then
          rx_byte_index_d1 <= rx_byte_index; 
        end if;
      end if;
    end if;
  end process;

--  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  --  -- 
  data_transfer_to_regs: process (clk_40m_i)
  begin
    if rising_edge (clk_40m_i) then
      if rst_i = '1' or rx_rst_i = '1' then
        word32_num                 <= 0;
        rx_frame_o                 <= (others =>(others =>'0'));
        else
          if rx_byte_ready_p = '1' and (rx_byte_index_d1) > 0 and (rx_byte_index_d1) mod 4 = 0 then
            word32_num             <= word32_num + 1;
            rx_frame_o(word32_num) <= word32;
          end if;
      end if;
    end if;
  end process;
  
  
end architecture struc;

--=================================================================================================
--                                        architecture end
--=================================================================================================
---------------------------------------------------------------------------------------------------
--                                      E N D   O F   F I L E
---------------------------------------------------------------------------------------------------